/*********************************************
**
**	Created on: 	08/04/2015 2015
**	Author: 	matt - Matt Edmunds
**	File:		Menu.cpp
**
**********************************************/

#include "Menu.h"
#include "ProjectData.h"
#include "Canvas.h"

Menu::Menu(ProjectData& data, Canvas& canvas, Enum::TreeType type, QObject *parent) : QObject(parent), mCanvas(canvas), mData(data)
{
	mIndex = 0;
	switch (type)
	{
		case Enum::TreeType::ROOT:
			root(parent);
			break;
		case Enum::TreeType::DATA:
			profile(parent);
			break;
		case Enum::TreeType::MESH:
			mesh(parent);
			break;
		case Enum::TreeType::FUNCTION:
			objective(parent);
			break;
		case Enum::TreeType::BOUNDARY:
			boundary(parent);
			break;
		case Enum::TreeType::OPTIMISER:
			optimiser(parent);
			break;
		case Enum::TreeType::RUNTIME:
			runtime(parent);
			break;
		case Enum::TreeType::NODE:
			node(parent);
			break;
		default:
			other(parent);
	}
}

Menu::~Menu()
{
	for (int i = 0; i < mActions.size(); ++i)
	{
		if (mActions.at(i)) delete mActions.at(i);
	}
}

void Menu::exec(const QPoint &pos)
{
	QMenu::exec(mActions, pos);
}

void Menu::exec(const QPoint& pos, const unsigned int& index)
{
	mIndex = index;
	QMenu::exec(mActions, pos);
}

void Menu::root(QObject *parent)
{
	QAction* loadObject = new QAction("Gosod Cyfeiriadur Prosiect", parent);
	QAction* clear = new QAction("Glirio Prosiect", parent);
	//TODO Save project here!
	//To solve end opt thread issue with files.

	mActions.append(loadObject);
	mActions.append(clear);

	QObject::connect(loadObject, SIGNAL( triggered() ), this->parent(), SLOT( loadProject() ));
	QObject::connect(clear, SIGNAL( triggered() ), this->parent(), SLOT( clearProject() ));
}

void Menu::profile(QObject *parent)
{
	QAction* importObject = new QAction("Mewnforio Proffil", parent);
	QAction* showObject = new QAction("Dangos Proffil", parent);
	QAction* hideObject = new QAction("Cuddio Proffil", parent);

	mActions.append(importObject);
	mActions.append(showObject);
	mActions.append(hideObject);

	QObject::connect(importObject, SIGNAL( triggered() ), this->parent(), SLOT( importProfile() ));
	QObject::connect(showObject, SIGNAL( triggered() ), this, SLOT( showProfile() ));
	QObject::connect(hideObject, SIGNAL( triggered() ), this, SLOT( hideProfile() ));
}

void Menu::mesh(QObject *parent)
{

    QAction* runmeshObject = new QAction("Rhedeg y Rhwyllwr", parent);
	QAction* showObject = new QAction("Dangos Rhwyll", parent);
	QAction* hideObject = new QAction("Cuddio Rhwyll", parent);

	mActions.append(runmeshObject);
	mActions.append(showObject);
	mActions.append(hideObject);

	QObject::connect(runmeshObject, SIGNAL( triggered() ), this->parent(), SLOT( runMesher() ));
	QObject::connect(showObject, SIGNAL( triggered() ), this, SLOT( showMesh() ));
	QObject::connect(hideObject, SIGNAL( triggered() ), this, SLOT( hideMesh() ));
}

void Menu::objective(QObject *parent)
{
	QAction* Object = new QAction("Gosod Ffwythiant Addasrwydd", parent);
	mActions.append(Object);
	QObject::connect(Object, SIGNAL( triggered() ), this->parent(), SLOT( setObjective() ));
}

void Menu::boundary(QObject *parent)
{
	QAction* Object = new QAction("Gosod Amodau Llif", parent);
	mActions.append(Object);
	QObject::connect(Object, SIGNAL( triggered() ), this->parent(), SLOT( setBoundary() ));
}

void Menu::optimiser(QObject *parent)
{
	QAction* Object = new QAction("Gosod Paramedrau'r Optimeiddiwr", parent);
	mActions.append(Object);
	QObject::connect(Object, SIGNAL( triggered() ), this->parent(), SLOT( setOptimiser() ));
}

void Menu::runtime(QObject *parent)
{
    QAction* runObject = new QAction("Rhedeg AerOpt", parent);
	QAction* showObject = new QAction("Dangos Graff", parent);
	mActions.append(runObject);
	mActions.append(showObject);
	QObject::connect(runObject, SIGNAL( triggered() ), this->parent(), SLOT( runAerOpt() ));
	QObject::connect(showObject, SIGNAL( triggered() ), this->parent(), SLOT( showGraph() ));
}

void Menu::other(QObject *parent)
{

}

void Menu::node(QObject *parent)
{
	QAction* setObject = new QAction("Gosod Terfynau", parent);
	QAction* unsetObject = new QAction("Ailgosod Terfynau", parent);

	mActions.append(setObject);
	mActions.append(unsetObject);

	QObject::connect(setObject, SIGNAL( triggered() ), this, SLOT( setConstraints() ));
	QObject::connect(unsetObject, SIGNAL( triggered() ), this, SLOT( resetConstraints() ));
}

void Menu::showProfile()
{
	mData.setRenderProfile(true);
	mCanvas.update();
}

void Menu::hideProfile()
{
	mData.setRenderProfile(false);
	mCanvas.update();
}

void Menu::showMesh()
{
	mData.setRenderMesh(true);
	mCanvas.update();
}

void Menu::hideMesh()
{
	mData.setRenderMesh(false);
	mCanvas.update();
}

void Menu::setConstraints()
{
	mCanvas.setConstraints(mIndex);
}

void Menu::resetConstraints()
{
	mCanvas.resetConstraints(mIndex);
}
